package ram.tutorial.editor;

import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentExtension3;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.DefaultCharacterPairMatcher;
import org.eclipse.jface.text.source.ICharacterPairMatcher;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.projection.ProjectionAnnotation;
import org.eclipse.jface.text.source.projection.ProjectionAnnotationModel;
import org.eclipse.jface.text.source.projection.ProjectionSupport;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.swt.custom.LineStyleEvent;
import org.eclipse.swt.custom.LineStyleListener;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.SourceViewerDecorationSupport;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import ram.tutorial.editor.outline.VGLOutlinePage;
import ram.tutorial.editor.parser.ASTStart;
import ram.tutorial.editor.util.EditorUtil;

/**
 * 
 * @author Ram Kulkarni (rakulkar@adobe.com)
 *
 */
public class VGLEditor extends TextEditor {

	VGLSourceViewConfiguration sourceViewerConfiguration  = null;
	
	VGLOutlinePage outlinePage = null;
	
	ASTStart startNode = null;
	
	ProjectionSupport projectionSupport = null;
	ProjectionAnnotationModel annotationModel = null;
	private Annotation[] oldAnnotations = null; 

	public static final String VGL_EDITOR_MATCHING_BRACKETS = "VGL_EDITOR_MATCHING_BRACKETS";
	public static final String VGL_EDITOR_MATCHING_BRACKETS_COLOR = "VGL_EDITOR_MATCHING_BRACKETS_COLOR";
	
	public VGLEditor() {
		super();
		sourceViewerConfiguration = new VGLSourceViewConfiguration(this);
	}

	public void createPartControl(Composite parent)
	{
		this.setSourceViewerConfiguration(sourceViewerConfiguration);
		super.createPartControl(parent);
		
	    ProjectionViewer viewer =(ProjectionViewer)getSourceViewer();

	    projectionSupport = new ProjectionSupport(viewer,getAnnotationAccess(),getSharedColors());
	    projectionSupport.install();

	    //turn projection mode on
	    viewer.doOperation(ProjectionViewer.TOGGLE);

	    annotationModel = viewer.getProjectionAnnotationModel();
		
		
		ISourceViewer sourceViewer = getSourceViewer();
		sourceViewer.getTextWidget().addLineStyleListener(new LineStyleListener (){

			public void lineGetStyle(LineStyleEvent event) {
				getLineStyle(event);
			}
		});
	}

	protected void getLineStyle(LineStyleEvent event) {
		
		if (startNode == null)
			return;
		
		int lineNum = -1;
		IDocument doc = getDocument();
		
		try
		{
			//because of projection viewer, convert widget offset to master doc offset
			int modelOffset = ((ProjectionViewer)getSourceViewer()).widgetOffset2ModelOffset(event.lineOffset);
			
			lineNum = doc.getLineOfOffset(modelOffset) + 1;

			if (lineNum < 1)
				return;
			
			ArrayList<StyleRange> styles = new ArrayList<StyleRange>();
			
			VGLColorizer.getInstance().colorizeLine(lineNum, modelOffset, startNode, styles);
			
			if (styles.size() > 0)
			{
				StyleRange[] styleArray = new StyleRange[styles.size()];
				
				for (int i = 0; i < styles.size(); i++)
				{
					styleArray[i] = styles.get(i);
					
					//convert master doc offset to widget offset
					styleArray[i].start = ((ProjectionViewer)getSourceViewer()).modelOffset2WidgetOffset(styleArray[i].start);
				}
				
				event.styles = styleArray;
			}
			
		} catch (Exception e) {}
		
	}

//	protected void getLineStyle(LineStyleEvent event) {
//		
//		if (startNode == null)
//			return;
//		
//		int lineNum = -1;
//		IDocument doc = getDocument();
//		
//		try
//		{
//			lineNum = doc.getLineOfOffset(event.lineOffset) + 1;
//
//			if (lineNum < 1)
//				return;
//			
//			ArrayList<StyleRange> styles = new ArrayList<StyleRange>();
//			
//			VGLColorizer.getInstance().colorizeLine(lineNum, event.lineOffset, startNode, styles);
//			
//			if (styles.size() > 0)
//			{
//				StyleRange[] styleArray = new StyleRange[styles.size()];
//				
//				for (int i = 0; i < styles.size(); i++)
//					styleArray[i] = styles.get(i);
//				
//				event.styles = styleArray;
//			}
//			
//		} catch (Exception e) {}
//		
//	}

	public Object getAdapter(Class required) {
		if (IContentOutlinePage.class.equals(required)) {
			if (outlinePage == null) {
				outlinePage= new VGLOutlinePage(getDocumentProvider(), this);
				if (getEditorInput() != null)
					outlinePage.setInput(this);
			}
			return outlinePage;
		}
		return super.getAdapter(required);
	}
	
	public IDocument getDocument()
	{
		return getSourceViewer().getDocument();
	}
	
	public ASTStart getStartNode()
	{
		if (startNode == null)
		{
			parseDocument();
			//refresh view
			redrawViewer();
		}
		return startNode;
	}
	
	public ASTStart parseDocument()
	{
		startNode = EditorUtil.parseDocument(getDocument());
		return startNode;
	}
	
	public void redrawViewer()
	{
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				//refresh view
				if ( getSourceViewer() != null)
					 getSourceViewer().getTextWidget().redraw();
			}
		});
		
	}
	
	protected ISourceViewer createSourceViewer(Composite parent,
            IVerticalRuler ruler, int styles)
	{
	   ISourceViewer viewer = new ProjectionViewer(parent, ruler,
					getOverviewRuler(), isOverviewRulerVisible(), styles);
	
	   // ensure decoration support has been created and configured.
	   getSourceViewerDecorationSupport(viewer);
	
	   return viewer;
	}

	public void updateFoldingStructure(ArrayList positions)
	{
		Annotation[] annotations = new Annotation[positions.size()];
		
		//this will hold the new annotations along
		//with their corresponding positions
		HashMap newAnnotations = new HashMap();
		
		for(int i =0;i<positions.size();i++)
		{
			ProjectionAnnotation annotation = new ProjectionAnnotation();
			
			newAnnotations.put(annotation,positions.get(i));
			
			annotations[i]=annotation;
		}
		
		annotationModel.modifyAnnotations(oldAnnotations,newAnnotations,null);
		
		oldAnnotations=annotations;
	}

	protected SourceViewerDecorationSupport getSourceViewerDecorationSupport(ISourceViewer viewer) {
		SourceViewerDecorationSupport decSupport = super.getSourceViewerDecorationSupport(viewer);

		char[] matchChars = {'(', ')', '[', ']', '{', '}' }; //which brackets to match		

		ICharacterPairMatcher matcher = new DefaultCharacterPairMatcher(matchChars ,
				IDocumentExtension3.DEFAULT_PARTITIONING);
		
		decSupport.setCharacterPairMatcher(matcher);

		decSupport.setMatchingCharacterPainterPreferenceKeys(VGL_EDITOR_MATCHING_BRACKETS,VGL_EDITOR_MATCHING_BRACKETS_COLOR);

		//Enable bracket highlighting in the preference store
		IPreferenceStore store = getPreferenceStore();
		store.setDefault(VGL_EDITOR_MATCHING_BRACKETS, true);
		store.setDefault(VGL_EDITOR_MATCHING_BRACKETS_COLOR, "128,128,128");
		
		return decSupport;
	}	
}
